export function encode(sequence) {
  const elements = Array.isArray(sequence) ? sequence : [sequence];
  const encoded = elements.map(n => encodeElement(n));
  if (encoded.length === 1) {
    return encoded[0];
  }
  return {
    precedence: 'sequence',
    pattern: encoded.map(n => n.precedence === 'disjunction' ? encodeAtomic(n) : n.pattern).join('')
  };
}
export function encodeAtomic(sequence) {
  const encoded = encode(sequence);
  return encoded.precedence === 'atom' ? encoded.pattern : `(?:${encoded.pattern})`;
}
function encodeElement(element) {
  if (typeof element === 'string') {
    return encodeText(element);
  }
  if (typeof element === 'object' && element instanceof RegExp) {
    return encodeRegExp(element);
  }
  if (typeof element === 'object' && 'pattern' in element) {
    return element;
  }
  if (typeof element === 'object' && 'chars' in element) {
    return encodeCharClass(element);
  }
  throw new Error(`\`encodeElement\`: unknown element: ${JSON.stringify(element, null, 2)}`);
}
function encodeText(text) {
  if (text.length === 0) {
    throw new Error('`encodeText`: received text should not be empty');
  }
  return {
    precedence: text.length === 1 ? 'atom' : 'sequence',
    pattern: escapeText(text)
  };
}
function encodeRegExp(regexp) {
  const pattern = regexp.source;
  return {
    precedence: isAtomicPattern(pattern) ? 'atom' : 'disjunction',
    pattern
  };
}
function isAtomicPattern(pattern) {
  if (pattern.length === 1) {
    return true;
  }
  if (pattern.startsWith('[') && pattern.endsWith(']') && pattern.match(/[[\]]/g)?.length === 2) {
    return true;
  }
  if (pattern.startsWith('(') && pattern.endsWith(')') && pattern.match(/[()]/g)?.length === 2) {
    return true;
  }
  return false;
}
export function encodeCharClass(element, isNegated) {
  if (!element.chars.length && !element.ranges?.length) {
    throw new Error('Character class should contain at least one character or character range');
  }
  const hyphen = element.chars.includes('-') ? '-' : '';
  const caret = element.chars.includes('^') ? '^' : '';
  const otherChars = element.chars.filter(c => c !== '-' && c !== '^').join('');
  const ranges = element.ranges?.map(({
    start,
    end
  }) => `${start}-${end}`).join('') ?? '';
  const negation = isNegated ? '^' : '';
  let pattern = `[${negation}${ranges}${otherChars}${caret}${hyphen}]`;
  if (pattern === '[^-]') pattern = '[\\^-]';
  return {
    precedence: 'atom',
    pattern
  };
}
function escapeText(text) {
  return text.replace(/[.*+?^${}()|[\]\\]/g, '\\$&');
}
//# sourceMappingURL=encoder.mjs.map